import 'package:flutter/cupertino.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/base/provider/request_info.dart';
import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestModel<T> extends ViewStateModel{

  T data;

  RequestInfo getRequestInfo();


  void initData({bool retry = false}) async{
    RequestInfo requestInfo = getRequestInfo();
    if (retry) {
      setBusy();
    }
    data = await DioUtils.getInstance().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: false,
      onFinish: (){
        refreshController.finishRefresh();
        refreshController.resetRefreshState();
      },
      onError: isInit? null : (e){
        setError(e, e.message, message: e.message);
      }
    ).then((res) => requestInfo?.handleRes(res.data));
    if (!isError) {
      setIdle();
    }
    if (isIdle && isEmpty(data)) {
      ///请求无异常但是数据为空需要设置为empty
      setEmpty();
    }
    onCompleted(data);
  }


  onCompleted(T data) {}


  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({Widget emptyWidget, ViewStateModelWidgetBuilder<T> builder}){
    return EasyRefresh(
      emptyWidget: emptyWidget??getRefreshNoDataWidget(viewStateModel: this),
      child: isNotEmpty(data)? builder(data) : Container(),
      onRefresh: isBusy? null : () async{
        initData();
      },
      controller: refreshController,
    );
  }
}

